/*****************************************************************************
 *
 * param.h
 *
 * PHASEX:  [P]hase [H]armonic [A]dvanced [S]ynthesis [EX]periment
 *
 * Copyright (C) 1999-2007 William Weston <weston@sysex.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *****************************************************************************/
#ifndef _PHASEX_PARAM_H_
#define _PHASEX_PARAM_H_

#include <gtk/gtk.h>
#include "phasex.h"
#include "gtkknob.h"


/* Parameter IDs */
#define PARAM_MIDI_CHANNEL		0
#define PARAM_MASTER_TUNE               1
#define PARAM_BPM			2
#define PARAM_VOLUME			3
#define PARAM_KEYMODE			4
#define PARAM_KEYFOLLOW_VOL		5
#define PARAM_TRANSPOSE			6
#define PARAM_PORTAMENTO		7
#define PARAM_INPUT_BOOST		8
#define PARAM_INPUT_FOLLOW		9
#define PARAM_STEREO_WIDTH		10

#define PARAM_AMP_ATTACK		11
#define PARAM_AMP_DECAY			12
#define PARAM_AMP_SUSTAIN		13
#define PARAM_AMP_RELEASE		14

#define PARAM_FILTER_CUTOFF		15
#define PARAM_FILTER_RESONANCE		16
#define PARAM_FILTER_SMOOTHING          17
#define PARAM_FILTER_KEYFOLLOW		18
#define PARAM_FILTER_MODE		19
#define PARAM_FILTER_TYPE		20
#define PARAM_FILTER_GAIN		21
#define PARAM_FILTER_ENV_AMOUNT		22
#define PARAM_FILTER_ENV_SIGN		23
#define PARAM_FILTER_LFO		24
#define PARAM_FILTER_LFO_AMOUNT		25
#define PARAM_FILTER_ATTACK		26
#define PARAM_FILTER_DECAY		27
#define PARAM_FILTER_SUSTAIN		28
#define PARAM_FILTER_RELEASE		29

#define PARAM_CHORUS_MIX		30
#define PARAM_CHORUS_AMOUNT		31
#define PARAM_CHORUS_TIME		32
#define PARAM_CHORUS_PHASE_RATE		33
#define PARAM_CHORUS_PHASE_AMOUNT	34
#define PARAM_CHORUS_FEED		35
#define PARAM_CHORUS_LFO_WAVE		36
#define PARAM_CHORUS_LFO_RATE		37
#define PARAM_CHORUS_FLIP		38

#define PARAM_DELAY_MIX			39
#define PARAM_DELAY_FEED		40
#define PARAM_DELAY_FLIP		41
#define PARAM_DELAY_TIME		42
#define PARAM_DELAY_LFO			43

#define PARAM_OSC1_MODULATION		44
#define PARAM_OSC1_WAVE			45
#define PARAM_OSC1_FREQ_BASE		46
#define PARAM_OSC1_RATE			47
#define PARAM_OSC1_POLARITY		48
#define PARAM_OSC1_INIT_PHASE		49
#define PARAM_OSC1_TRANSPOSE		50
#define PARAM_OSC1_PITCHBEND		51
#define PARAM_OSC1_AM_LFO		52
#define PARAM_OSC1_AM_LFO_AMOUNT	53
#define PARAM_OSC1_FREQ_LFO		54
#define PARAM_OSC1_FREQ_LFO_AMOUNT	55
#define PARAM_OSC1_PHASE_LFO		56
#define PARAM_OSC1_PHASE_LFO_AMOUNT	57
#define PARAM_OSC1_WAVE_LFO		58
#define PARAM_OSC1_WAVE_LFO_AMOUNT	59

#define PARAM_OSC2_MODULATION		60
#define PARAM_OSC2_WAVE			61
#define PARAM_OSC2_FREQ_BASE		62
#define PARAM_OSC2_RATE			63
#define PARAM_OSC2_POLARITY		64
#define PARAM_OSC2_INIT_PHASE		65
#define PARAM_OSC2_TRANSPOSE		66
#define PARAM_OSC2_PITCHBEND		67
#define PARAM_OSC2_AM_LFO		68
#define PARAM_OSC2_AM_LFO_AMOUNT	69
#define PARAM_OSC2_FREQ_LFO		70
#define PARAM_OSC2_FREQ_LFO_AMOUNT	71
#define PARAM_OSC2_PHASE_LFO		72
#define PARAM_OSC2_PHASE_LFO_AMOUNT	73
#define PARAM_OSC2_WAVE_LFO		74
#define PARAM_OSC2_WAVE_LFO_AMOUNT	75

#define PARAM_OSC3_MODULATION		76
#define PARAM_OSC3_WAVE			77
#define PARAM_OSC3_FREQ_BASE		78
#define PARAM_OSC3_RATE			79
#define PARAM_OSC3_POLARITY		80
#define PARAM_OSC3_INIT_PHASE		81
#define PARAM_OSC3_TRANSPOSE		82
#define PARAM_OSC3_PITCHBEND		83
#define PARAM_OSC3_AM_LFO		84
#define PARAM_OSC3_AM_LFO_AMOUNT	85
#define PARAM_OSC3_FREQ_LFO		86
#define PARAM_OSC3_FREQ_LFO_AMOUNT	87
#define PARAM_OSC3_PHASE_LFO		88
#define PARAM_OSC3_PHASE_LFO_AMOUNT	89
#define PARAM_OSC3_WAVE_LFO		90
#define PARAM_OSC3_WAVE_LFO_AMOUNT	91

#define PARAM_OSC4_MODULATION		92
#define PARAM_OSC4_WAVE			93
#define PARAM_OSC4_FREQ_BASE		94
#define PARAM_OSC4_RATE			95
#define PARAM_OSC4_POLARITY		96
#define PARAM_OSC4_INIT_PHASE		97
#define PARAM_OSC4_TRANSPOSE		98
#define PARAM_OSC4_PITCHBEND		99
#define PARAM_OSC4_AM_LFO		100
#define PARAM_OSC4_AM_LFO_AMOUNT	101
#define PARAM_OSC4_FREQ_LFO		102
#define PARAM_OSC4_FREQ_LFO_AMOUNT	103
#define PARAM_OSC4_PHASE_LFO		104
#define PARAM_OSC4_PHASE_LFO_AMOUNT	105
#define PARAM_OSC4_WAVE_LFO		106
#define PARAM_OSC4_WAVE_LFO_AMOUNT	107

#define PARAM_LFO1_WAVE			108
#define PARAM_LFO1_FREQ_BASE		109
#define PARAM_LFO1_RATE			110
#define PARAM_LFO1_POLARITY		111
#define PARAM_LFO1_INIT_PHASE		112
#define PARAM_LFO1_TRANSPOSE		113
#define PARAM_LFO1_PITCHBEND		114

#define PARAM_LFO2_WAVE			115
#define PARAM_LFO2_FREQ_BASE		116
#define PARAM_LFO2_RATE			117
#define PARAM_LFO2_POLARITY		118
#define PARAM_LFO2_INIT_PHASE		119
#define PARAM_LFO2_TRANSPOSE		120
#define PARAM_LFO2_PITCHBEND		121

#define PARAM_LFO3_WAVE			122
#define PARAM_LFO3_FREQ_BASE		123
#define PARAM_LFO3_RATE			124
#define PARAM_LFO3_POLARITY		125
#define PARAM_LFO3_INIT_PHASE		126
#define PARAM_LFO3_TRANSPOSE		127
#define PARAM_LFO3_PITCHBEND		128

#define PARAM_LFO4_WAVE			129
#define PARAM_LFO4_FREQ_BASE		130
#define PARAM_LFO4_RATE			131
#define PARAM_LFO4_POLARITY		132
#define PARAM_LFO4_INIT_PHASE		133
#define PARAM_LFO4_TRANSPOSE		134
#define PARAM_LFO4_PITCHBEND		135

/* Update NUM_PARAMS after adding or removing parameters */
#define NUM_PARAMS			136

/* The following only behave like parameters for the help system */
#define PARAM_PROGRAM_NUMBER		136
#define PARAM_PATCH_NAME		137
#define PARAM_BANK_MEM_MODE		138

/* Main help for PHASEX */
#define PARAM_PHASEX_HELP		139

/* Update NUM_HELP_PARAMS after adding or removing parameters */
#define NUM_HELP_PARAMS			140

/* Update NUM_PARAM_GROUPS after changing groups in param.c */
#define NUM_PARAM_GROUPS		22

/* Update NUM_PARAM_PAGES after changing pages in param.c */
#define NUM_PARAM_PAGES			2

/* Parameter types */
#define PARAM_TYPE_HELP			0
#define PARAM_TYPE_INT			1
#define PARAM_TYPE_REAL			2
#define PARAM_TYPE_LIST			3
#define PARAM_TYPE_BOOL			4
#define PARAM_TYPE_RATE			5
#define PARAM_TYPE_BBOX			6
#define PARAM_TYPE_DTNT			7


typedef void (*PHASEX_CALLBACK) (GtkWidget *widget, gpointer *data);


/* Parameter struct for handling conversion and value passing */
typedef struct phasex_param {
    const char		*name;		/* Parameter name for patches		*/
    const char		*label;		/* Parameter label for GUI		*/
    int			id;		/* Param unique ID.  See defs above.	*/
    int			type;		/* Integer, real, or list values	*/
    int			cc_num;		/* MIDI controller number		*/
    int			cc_limit;	/* Upper bound for MIDI controller	*/
    int			leap;		/* leap (moderately large step) size	*/
    int			pre_inc;	/* Pre-scaling constant adjustment	*/
    int			index;		/* Index for arrayed parameters		*/
    int			updated;	/* Flag set by non-gui threads		*/
    int			locked;		/* Allow only user-explicit updates	*/
    int			cc_cur;		/* Current  MIDI ctlr value		*/
    int			cc_prev;	/* Previous MIDI ctlr value		*/
    int			cc_val[NUM_PARAMS];	/* Current MIDI ctlr value	*/
    int			int_val[NUM_PARAMS];	/* Current integer param value	*/
    PHASEX_CALLBACK	callback;	/* Universal parameter update callback	*/
    GtkKnob		*knob;		/* Widget that holds param value	*/
    GtkObject		*adj;		/* Widget that holds param value	*/
    GtkWidget		*spin;		/* Widget that holds param value	*/
    GtkWidget		*combo;		/* Widget that holds param value	*/
    GtkWidget		*text;		/* Widget that holds param value	*/
    GtkWidget		*button[12];	/* Widgets that hold param value	*/
    const gchar		**list_labels;	/* List for list based parameters	*/
} PARAM;

/* Parameter group, with array of parameter IDs */
typedef struct param_group {
    gchar		*label;		/* Frame label for parameter group	*/
    int			full_x;		/* Table column in fullscreen layout	*/
    int			full_y0;	/* Table row start in fullscreen layout	*/
    int			full_y1;	/* Table row end in fullscreen layout	*/
    int			notebook_page;	/* Tab number, starting at 0		*/
    int			notebook_x;	/* Table column in notebook layout	*/
    int			notebook_y0;	/* Table row start in notebook layout	*/
    int			notebook_y1;	/* Table row end in notebook layout	*/
    int			param_list[15];	/* List of up to 14 parameters		*/
} PARAM_GROUP;

/* Parameter page definition */
typedef struct param_page {
    gchar		*label;		/* Label for GUI page			*/
} PARAM_PAGE;


/* List of labels for parameter values in GUI */
extern const char *on_off_labels[];
extern const char *freq_base_labels[];
extern const char *mod_type_labels[];
extern const char *keymode_labels[];
extern const char *keyfollow_labels[];
extern const char *polarity_labels[];
extern const char *sign_labels[];
extern const char *wave_labels[];
extern const char *filter_mode_labels[];
extern const char *lfo_labels[];
extern const char *rate_labels[];


/* All parameters belong in this array */
PARAM param[NUM_HELP_PARAMS];

/* Parameter groups and pages reference paramenters */
PARAM_GROUP param_group[NUM_PARAM_GROUPS];
PARAM_PAGE param_page[NUM_PARAM_PAGES];

char *midimap_filename;
int midimap_modified;


/* Functions to initialize global arrays */
void init_param(int id, const char *name, const char *label, int type,
		int cc, int lim, int ccv, int pre, int ix, int leap,
		PHASEX_CALLBACK callback, const char *label_list[]);
void init_params(void);
void init_param_groups(void);
void init_param_pages(void);
int read_midimap(char *filename);
int save_midimap(char *filename);


#endif /* _PHASEX_PARAM_H_ */
